@@ -6,31 +6,24 @@ import shutil
 import tempfile
 import unittest
 import site
-from setuptools.compat import StringIO, BytesIO, next
-from setuptools.compat import urlparse
+import contextlib
 import textwrap
 import tarfile
+import logging
 import distutils.core
 
 from setuptools.compat import StringIO, BytesIO, next, urlparse
 from setuptools.sandbox import run_setup, SandboxViolation
-from setuptools.command.easy_install import easy_install, fix_jython_executable, get_script_args, nt_quote_arg
-from setuptools.command.easy_install import  PthDistributions
+from setuptools.command.easy_install import (
+    easy_install, fix_jython_executable, get_script_args, nt_quote_arg)
+from setuptools.command.easy_install import PthDistributions
 from setuptools.command import easy_install as easy_install_pkg
 from setuptools.dist import Distribution
+from pkg_resources import working_set, VersionConflict
 from pkg_resources import Distribution as PRDistribution
 import setuptools.tests.server
-
-try:
-    # import multiprocessing solely for the purpose of testing its existence
-    __import__('multiprocessing')
-    import logging
-    _LOG = logging.getLogger('test_easy_install')
-    logging.basicConfig(level=logging.INFO, stream=sys.stderr)
-    _MULTIPROC = True
-except ImportError:
-    _MULTIPROC = False
-    _LOG = None
+import pkg_resources
+from .py26compat import skipIf
 
 class FakeDist(object):
     def get_entry_map(self, group):
@@ -140,31 +133,29 @@ class TestUserInstallTest(unittest.TestCase):
         f.close()
         self.old_cwd = os.getcwd()
         os.chdir(self.dir)
-        if sys.version >= "2.6":
-            self.old_has_site = easy_install_pkg.HAS_USER_SITE
-            self.old_file = easy_install_pkg.__file__
-            self.old_base = site.USER_BASE
-            site.USER_BASE = tempfile.mkdtemp()
-            self.old_site = site.USER_SITE
-            site.USER_SITE = tempfile.mkdtemp()
-            easy_install_pkg.__file__ = site.USER_SITE
+
+        self.old_enable_site = site.ENABLE_USER_SITE
+        self.old_file = easy_install_pkg.__file__
+        self.old_base = site.USER_BASE
+        site.USER_BASE = tempfile.mkdtemp()
+        self.old_site = site.USER_SITE
+        site.USER_SITE = tempfile.mkdtemp()
+        easy_install_pkg.__file__ = site.USER_SITE
 
     def tearDown(self):
         os.chdir(self.old_cwd)
         shutil.rmtree(self.dir)
-        if sys.version >= "2.6":
-            shutil.rmtree(site.USER_BASE)
-            shutil.rmtree(site.USER_SITE)
-            site.USER_BASE = self.old_base
-            site.USER_SITE = self.old_site
-            easy_install_pkg.HAS_USER_SITE = self.old_has_site
-            easy_install_pkg.__file__ = self.old_file
+
+        shutil.rmtree(site.USER_BASE)
+        shutil.rmtree(site.USER_SITE)
+        site.USER_BASE = self.old_base
+        site.USER_SITE = self.old_site
+        site.ENABLE_USER_SITE = self.old_enable_site
+        easy_install_pkg.__file__ = self.old_file
 
     def test_user_install_implied(self):
-        easy_install_pkg.HAS_USER_SITE = True # disabled sometimes
+        site.ENABLE_USER_SITE = True # disabled sometimes
         #XXX: replace with something meaningfull
-        if sys.version < "2.6":
-            return #SKIP
         dist = Distribution()
         dist.script_name = 'setup.py'
         cmd = easy_install(dist)
@@ -173,15 +164,19 @@ class TestUserInstallTest(unittest.TestCase):
         self.assertTrue(cmd.user, 'user should be implied')
 
     def test_multiproc_atexit(self):
-        if not _MULTIPROC:
+        try:
+            __import__('multiprocessing')
+        except ImportError:
+            # skip the test if multiprocessing is not available
             return
-        _LOG.info('this should not break')
+
+        log = logging.getLogger('test_easy_install')
+        logging.basicConfig(level=logging.INFO, stream=sys.stderr)
+        log.info('this should not break')
 
     def test_user_install_not_implied_without_usersite_enabled(self):
-        easy_install_pkg.HAS_USER_SITE = False # usually enabled
+        site.ENABLE_USER_SITE = False # usually enabled
         #XXX: replace with something meaningfull
-        if sys.version < "2.6":
-            return #SKIP
         dist = Distribution()
         dist.script_name = 'setup.py'
         cmd = easy_install(dist)
@@ -214,8 +209,9 @@ class TestUserInstallTest(unittest.TestCase):
             cmd.ensure_finalized()
             cmd.local_index.scan([new_location])
             res = cmd.easy_install('foo')
-            self.assertEqual(os.path.realpath(res.location),
-                             os.path.realpath(new_location))
+            actual = os.path.normcase(os.path.realpath(res.location))
+            expected = os.path.normcase(os.path.realpath(new_location))
+            self.assertEqual(actual, expected)
         finally:
             sys.path.remove(target)
             for basedir in [new_location, target, ]:
@@ -230,6 +226,9 @@ class TestUserInstallTest(unittest.TestCase):
             else:
                 del os.environ['PYTHONPATH']
 
+    @skipIf(sys.version_info < (3, 4),
+        "Test fails on Python 3.3 and earlier due to bug in inspect but only "
+        "when run under setup.py test")
     def test_setup_requires(self):
         """Regression test for Distribute issue #318
 
@@ -238,49 +237,15 @@ class TestUserInstallTest(unittest.TestCase):
         SandboxViolation.
         """
 
-        test_setup_attrs = {
-            'name': 'test_pkg', 'version': '0.0',
-            'setup_requires': ['foobar'],
-            'dependency_links': [os.path.abspath(self.dir)]
-        }
-
-        test_pkg = os.path.join(self.dir, 'test_pkg')
+        test_pkg = create_setup_requires_package(self.dir)
         test_setup_py = os.path.join(test_pkg, 'setup.py')
-        test_setup_cfg = os.path.join(test_pkg, 'setup.cfg')
-        os.mkdir(test_pkg)
 
-        f = open(test_setup_py, 'w')
-        f.write(textwrap.dedent("""\
-            import setuptools
-            setuptools.setup(**%r)
-        """ % test_setup_attrs))
-        f.close()
-
-        foobar_path = os.path.join(self.dir, 'foobar-0.1.tar.gz')
-        make_trivial_sdist(
-            foobar_path,
-            textwrap.dedent("""\
-                import setuptools
-                setuptools.setup(
-                    name='foobar',
-                    version='0.1'
-                )
-            """))
-
-        old_stdout = sys.stdout
-        old_stderr = sys.stderr
-        sys.stdout = StringIO()
-        sys.stderr = StringIO()
         try:
-            try:
-                reset_setup_stop_context(
-                    lambda: run_setup(test_setup_py, ['install'])
-                )
-            except SandboxViolation:
-                self.fail('Installation caused SandboxViolation')
-        finally:
-            sys.stdout = old_stdout
-            sys.stderr = old_stderr
+            with quiet_context():
+                with reset_setup_stop_context():
+                    run_setup(test_setup_py, ['install'])
+        except SandboxViolation:
+            self.fail('Installation caused SandboxViolation')
 
 
 class TestSetupRequires(unittest.TestCase):
@@ -300,45 +265,34 @@ class TestSetupRequires(unittest.TestCase):
             # Some platforms (Jython) don't find a port to which to bind,
             #  so skip this test for them.
             return
-
-        # I realize this is all-but-impossible to read, because it was
-        #  ported from some well-factored, safe code using 'with'. If you
-        #  need to maintain this code, consider making the changes in
-        #  the parent revision (of this comment) and then port the changes
-        #  back for Python 2.4 (or deprecate Python 2.4).
-
-        def install(dist_file):
-            def install_at(temp_install_dir):
-                def install_env():
-                    ei_params = ['--index-url', p_index.url,
-                        '--allow-hosts', p_index_loc,
-                        '--exclude-scripts', '--install-dir', temp_install_dir,
-                        dist_file]
-                    def install_clean_reset():
-                        def install_clean_argv():
-                            # attempt to install the dist. It should fail because
-                            #  it doesn't exist.
-                            self.assertRaises(SystemExit,
-                                easy_install_pkg.main, ei_params)
-                        argv_context(install_clean_argv, ['easy_install'])
-                    reset_setup_stop_context(install_clean_reset)
-                environment_context(install_env, PYTHONPATH=temp_install_dir)
-            tempdir_context(install_at)
-
-        # create an sdist that has a build-time dependency.
-        self.create_sdist(install)
-
+        with quiet_context():
+            # create an sdist that has a build-time dependency.
+            with TestSetupRequires.create_sdist() as dist_file:
+                with tempdir_context() as temp_install_dir:
+                    with environment_context(PYTHONPATH=temp_install_dir):
+                        ei_params = ['--index-url', p_index.url,
+                            '--allow-hosts', p_index_loc,
+                            '--exclude-scripts', '--install-dir', temp_install_dir,
+                            dist_file]
+                        with reset_setup_stop_context():
+                            with argv_context(['easy_install']):
+                                # attempt to install the dist. It should fail because
+                                #  it doesn't exist.
+                                self.assertRaises(SystemExit,
+                                    easy_install_pkg.main, ei_params)
         # there should have been two or three requests to the server
         #  (three happens on Python 3.3a)
         self.assertTrue(2 <= len(p_index.requests) <= 3)
         self.assertEqual(p_index.requests[0].path, '/does-not-exist/')
 
-    def create_sdist(self, installer):
+    @staticmethod
+    @contextlib.contextmanager
+    def create_sdist():
         """
-        Create an sdist with a setup_requires dependency (of something that
-        doesn't exist) and invoke installer on it.
+        Return an sdist with a setup_requires dependency (of something that
+        doesn't exist)
         """
-        def build_sdist(dir):
+        with tempdir_context() as dir:
             dist_path = os.path.join(dir, 'setuptools-test-fetcher-1.0.tar.gz')
             make_trivial_sdist(
                 dist_path,
@@ -350,8 +304,78 @@ class TestSetupRequires(unittest.TestCase):
                         setup_requires = ['does-not-exist'],
                     )
                 """).lstrip())
-            installer(dist_path)
-        tempdir_context(build_sdist)
+            yield dist_path
+
+    def test_setup_requires_overrides_version_conflict(self):
+        """
+        Regression test for issue #323.
+
+        Ensures that a distribution's setup_requires requirements can still be
+        installed and used locally even if a conflicting version of that
+        requirement is already on the path.
+        """
+
+        pr_state = pkg_resources.__getstate__()
+        fake_dist = PRDistribution('does-not-matter', project_name='foobar',
+                                   version='0.0')
+        working_set.add(fake_dist)
+
+        try:
+            with tempdir_context() as temp_dir:
+                test_pkg = create_setup_requires_package(temp_dir)
+                test_setup_py = os.path.join(test_pkg, 'setup.py')
+                with quiet_context() as (stdout, stderr):
+                    with reset_setup_stop_context():
+                        try:
+                            # Don't even need to install the package, just
+                            # running the setup.py at all is sufficient
+                            run_setup(test_setup_py, ['--name'])
+                        except VersionConflict:
+                            self.fail('Installing setup.py requirements '
+                                'caused a VersionConflict')
+
+                lines = stdout.readlines()
+                self.assertTrue(len(lines) > 0)
+                self.assertTrue(lines[-1].strip(), 'test_pkg')
+        finally:
+            pkg_resources.__setstate__(pr_state)
+
+
+def create_setup_requires_package(path):
+    """Creates a source tree under path for a trivial test package that has a
+    single requirement in setup_requires--a tarball for that requirement is
+    also created and added to the dependency_links argument.
+    """
+
+    test_setup_attrs = {
+        'name': 'test_pkg', 'version': '0.0',
+        'setup_requires': ['foobar==0.1'],
+        'dependency_links': [os.path.abspath(path)]
+    }
+
+    test_pkg = os.path.join(path, 'test_pkg')
+    test_setup_py = os.path.join(test_pkg, 'setup.py')
+    os.mkdir(test_pkg)
+
+    f = open(test_setup_py, 'w')
+    f.write(textwrap.dedent("""\
+        import setuptools
+        setuptools.setup(**%r)
+    """ % test_setup_attrs))
+    f.close()
+
+    foobar_path = os.path.join(path, 'foobar-0.1.tar.gz')
+    make_trivial_sdist(
+        foobar_path,
+        textwrap.dedent("""\
+            import setuptools
+            setuptools.setup(
+                name='foobar',
+                version='0.1'
+            )
+        """))
+
+    return test_pkg
 
 
 def make_trivial_sdist(dist_path, setup_py):
@@ -374,44 +398,37 @@ def make_trivial_sdist(dist_path, setup_py):
         dist.close()
 
 
-def tempdir_context(f, cd=lambda dir:None):
-    """
-    Invoke f in the context
-    """
+@contextlib.contextmanager
+def tempdir_context(cd=lambda dir:None):
     temp_dir = tempfile.mkdtemp()
     orig_dir = os.getcwd()
     try:
         cd(temp_dir)
-        f(temp_dir)
+        yield temp_dir
     finally:
         cd(orig_dir)
         shutil.rmtree(temp_dir)
 
-def environment_context(f, **updates):
-    """
-    Invoke f in the context
-    """
+@contextlib.contextmanager
+def environment_context(**updates):
     old_env = os.environ.copy()
     os.environ.update(updates)
     try:
-        f()
+        yield
     finally:
         for key in updates:
             del os.environ[key]
         os.environ.update(old_env)
 
-def argv_context(f, repl):
-    """
-    Invoke f in the context
-    """
+@contextlib.contextmanager
+def argv_context(repl):
     old_argv = sys.argv[:]
     sys.argv[:] = repl
-    try:
-        f()
-    finally:
-        sys.argv[:] = old_argv
+    yield
+    sys.argv[:] = old_argv
 
-def reset_setup_stop_context(f):
+@contextlib.contextmanager
+def reset_setup_stop_context():
     """
     When the setuptools tests are run using setup.py test, and then
     one wants to invoke another setup() command (such as easy_install)
@@ -420,7 +437,25 @@ def reset_setup_stop_context(f):
     """
     setup_stop_after = distutils.core._setup_stop_after
     distutils.core._setup_stop_after = None
+    yield
+    distutils.core._setup_stop_after = setup_stop_after
+
+
+@contextlib.contextmanager
+def quiet_context():
+    """
+    Redirect stdout/stderr to StringIO objects to prevent console output from
+    distutils commands.
+    """
+
+    old_stdout = sys.stdout
+    old_stderr = sys.stderr
+    new_stdout = sys.stdout = StringIO()
+    new_stderr = sys.stderr = StringIO()
     try:
-        f()
+        yield new_stdout, new_stderr
     finally:
-        distutils.core._setup_stop_after = setup_stop_after
+        new_stdout.seek(0)
+        new_stderr.seek(0)
+        sys.stdout = old_stdout
+        sys.stderr = old_stderr
